@@ -5,13 +5,13 @@ import importlib
 from watchdog.observers import Observer
 from watchdog.events import FileSystemEventHandler
 from openhab_pythonrule_engine.item_registry import ItemRegistry
+from openhab_pythonrule_engine.processor import Processor
 from openhab_pythonrule_engine.cron_processor import CronProcessor
 from openhab_pythonrule_engine.item_change_processor import ItemChangeProcessor
 from openhab_pythonrule_engine.loaded_rule_processor import RuleLoadedProcessor
 from openhab_pythonrule_engine.source_scanner import visit
 
 
-
 class FileSystemListener(FileSystemEventHandler):
 
     def __init__(self, rule_engine, dir):
@@ -57,15 +57,36 @@ class RuleEngine:
         self.openhab_uri = openhab_uri
         logging.info("connecting " + openhab_uri)
         self.loaded_modules = set()
+        self.last_executed = ""
+        self.last_error = ""
         self.__item_registry = ItemRegistry(openhab_uri, user, pwd)
-        self.__processors = [ItemChangeProcessor(openhab_uri, self.__item_registry),
-                             CronProcessor(self.__item_registry),
-                             RuleLoadedProcessor(self.__item_registry)]
+        self.__processors = [ItemChangeProcessor(openhab_uri, self.__item_registry, self.on_executed),
+                             CronProcessor(self.__item_registry, self.on_executed),
+                             RuleLoadedProcessor(self.__item_registry, self.on_executed)]
         self.file_system_listener = FileSystemListener(self, python_rule_directory)
+        self.listeners = set()
+
+    def on_executed(self, source: Processor, success: bool):
+        if success:
+            self.last_executed = source.last_executed
+        else:
+            self.last_executed = source.last_error
 
     def __del__(self):
         self.stop()
 
+    def add_listener(self, listener):
+        self.listeners.add(listener)
+        self.__notify_listener()
+
+
+    def __notify_listener(self):
+        for listener in self.listeners:
+            try:
+                listener()
+            except Exception as e:
+                logging.warning("error occurred calling " + str(listener) + " " + str(e))
+
     def start(self):
         if not self.is_running:
             self.is_running = True
@@ -97,6 +118,7 @@ class RuleEngine:
                     importlib.import_module(modulename)
                     msg = "'" + filename + "' loaded for the first time"
                 self.loaded_modules.add(filename)
+                self.__notify_listener()
                 num_annotations = visit(modulename, [processor.parser() for processor in self.__processors])
                 if num_annotations > 0:
                     logging.info(msg)
@@ -113,6 +135,7 @@ class RuleEngine:
                     [processor.remove_triggers(modulename) for processor in self.__processors]
                     del sys.modules[modulename]
                 self.loaded_modules.remove(filename)
+                self.__notify_listener()
             except Exception as e:
                 logging.warning("error occurred by unloading " + filename + " " + str(e), e)
 
